{ +--------------------------------------------------------------------------+ }
{ | Tubes2 for DOS 0.2 Open edition * Electron tube catalog                  | }
{ | Copyright (C) 2016-2017 Pozsar Zsolt <info@pozsarzs.hu>                  | }
{ | tubes2oe.pas                                                             | }
{ | program file                                                             | }
{ +--------------------------------------------------------------------------+ }

{
   This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by the
 Free Software Foundation, either version 3 of the License, or (at your
 option) any later version.

   This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

   You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$MODE OBJFPC}{$H+}
program tubes2oe;
uses
  crt, dos, gettext,
  ucatalog, ucommon, uconfig;
var
  appmode: byte;
const
  params: array[1..2,1..3] of string=
  (
    ('-h','--help','show help'),
    ('-v','--version','show version and build information')
  );

// -- show useable parameters -------------------------------------------------
procedure help(mode: boolean);
var
  b: byte;
begin
  if mode then
    writeln('There are one or more bad parameter in command line.') else
    begin
      writeln('Usage: tubes2oe.exe [parameter]');
      writeln;
      writeln('parameters:');
      for b:=1 to 2 do
      begin
        write('  ',params[b,1]);
        gotoxy(8,wherey); write(params[b,2]);
        gotoxy(30,wherey); writeln(params[b,3]);
      end;
      writeln;
    end;
  halt(0);
end;

// -- show version and build information --------------------------------------
procedure verinfo;
begin
  writeln(ucommon.PRGNAME+' v'+ucommon.PRGVER);
  writeln;
  writeln('This application was compiled at ',{$I %TIME%},' on ',{$I %DATE%},
    ' by pozsarzs.');
  writeln('FPC version: ',{$I %FPCVERSION%});
  writeln('Target OS:   ',{$I %FPCTARGETOS%});
  writeln('Target CPU:  ',{$I %FPCTARGETCPU%});
  halt(0);
end;

// -- main program -------------------------------------------------------------
begin
  setexepath;

  // detect language
  lng:=getenv('LANG');
  if length(lng)=0 then lng:='en';
  lng:=lowercase(lng[1]+lng[2]);

  // parameters
  appmode:=0;
  if length(paramstr(1))<>0 then
  begin
    for b:=1 to 2 do
    begin
      if paramstr(1)=params[b,1] then appmode:=b+1;
      if paramstr(1)=params[b,2] then appmode:=b+1;
    end;
    case appmode of
      2: help(false);
      3: verinfo;
    else help(true);
    end;
  end;

  // load configuration
  if loadcfgfile(exepath+'..\config\tubes2oe.cfg')=false then
  begin
    writeln('Cannot open configuration file!');
    halt(1);
  end;

  // translate messages
  translateresourcestrings(exepath+'..\language\tubes_'+lng+'.mo');

  // load palette
  if loadpalfile(exepath+'..\palette\'+fpal)=false then
  begin
    writeln('Cannot open palette file!');
    halt(2);
  end;

  // start catalog
  cursoroff;
  catalog_pages;
  quit(0,'');
end.
