{ +--------------------------------------------------------------------------+ }
{ | Tubes2 for DOS 0.2 Open edition * Electron tube catalog                  | }
{ | Copyright (C) 2016-2017 Pozsar Zsolt <info@pozsarzs.hu>                  | }
{ | ucatalog.pas                                                             | }
{ | catalog page viewer                                                      | }
{ +--------------------------------------------------------------------------+ }

{
   This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by the
 Free Software Foundation, either version 3 of the License, or (at your
 option) any later version.

   This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

   You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$MODE OBJFPC}{$H+}
unit ucatalog;
interface
uses
  crt, dos, printer,
  ucommon, uconfig, uhelp, usubst;
type
  tcpct=array[1..6000] of byte;
  tcpnm=array[1..6000] of string[16];
  tctgr=array[1..64] of string[50];
  ta_cpld=array[1..90] of string[44];
  ta_cppn=array[1..48] of string[23];
  tdatafilebuffer=array[0..65535] of byte;
var
  // arrays in heap
  cpct: ^tcpct;                                       { Category of components }
  cpnm: ^tcpnm;                                          { Names of components }
  ctgr: ^tctgr;                                                   { Categories }
  a_cpld: ^ta_cpld;                     { Long description of actual component }
  a_cppn: ^ta_cppn;                         { Pin function of actual component }
  dfb: ^tdatafilebuffer;                       { Read buffer for load datafile }
  // others
  a_cpct: string[64];                           { Category of actual component }
  a_cpds: string[64];                  { Short description of actual component }
  a_cpnm: string[16];                               { Name of actual component }
  a_cppc: string[16];                            { Package of actual component }
  b,bb: byte;                                         { General byte variables }
  c: char;                                                 { General character }
  dscr: string[32];                              { Description of catalog file }
  headtext: string[78];                                       { Text of header }
  i,ii,iii: integer;                               { General integer variables }
  lastcomp: integer;              { Number of last component in actual catalog }
  ldend,pend: byte;                     { End of long description and pin list }
  ldp, ldpmax, pp, ppmax: byte;                                 { Page numbers }
  messtext: messagerec;                      { Variable of messages procedures }
  s, ss: string;                                     { General string variable }
  datafileversion: string[9];                               { Datafile version }

procedure catalog_pages;

resourcestring
 MESSAGE01 = 'Warning';
 MESSAGE02 = 'Load type list';
 MESSAGE03 = '<Enter> OK';
 MESSAGE04 = 'There aren''t any database files!';
 MESSAGE05 = '<F1> help  <F2> choice  <F3> type  <F4> view base  <F10> menu';
 MESSAGE06 = 'electron tube catalog';
 MESSAGE07 = 'Error';
 MESSAGE08 = '    No such catalog!';
 MESSAGE09 = 'Search';
 MESSAGE10 = 'Cancel';
 MESSAGE11 = '<Enter> accept  <Escape> cancel';
 MESSAGE12 = 'Failed to open catalog file!';
 MESSAGE13 = 'Failed to create output  file!';
 MESSAGE14 = 'Save';
 MESSAGE15 = 'File name without ext.:';
 MESSAGE16 = 'Type:';
 MESSAGE17 = 'Function:';
 MESSAGE18 = 'Base type:';
 MESSAGE19 = 'Pin-out:';
 MESSAGE20 = 'Description:';
 MESSAGE21 = 'Printer error!';
 MESSAGE22 = '<Arrows> move  <Enter> choose  <Escape> back';
 MESSAGE23 = '<Home End C-Home C-End PgUp PgDn Arrows> move  <Enter> choose  <Escape> back';
 MESSAGE24 = 'It isn''t useable in this        category!';
 MESSAGE25 = 'View tube <b>ase                F4';
 MESSAGE26 = ' Cannot run ext. viewer!    Check tubes2oe.cfg.';
 {...}
 MESSAGE32 = 'Cho<i>ce                        F2';
 MESSAGE33 = 'T<y>pe                          F3';
 MESSAGE34 = 'S<u>bstitution';
 MESSAGE35 = 'DOS she<l>l';
 MESSAGE36 = 'Type ''exit'' to return program.';
 MESSAGE37 = 'Cannot run DOS shell!';
 MESSAGE38 = 'Search and load';
 MESSAGE39 = '<P>ackage information';
 MESSAGE40 = '<Enter> accept  <Escape> show next';
 MESSAGE41 = 'More results';
 MESSAGE42 = 'Result';
 MESSAGE43 = 'Load';
 MESSAGE44 = 'Next';
 MESSAGE45 = '<H>elp content';
 MESSAGE46 = '<S>ave this page            Ctrl-s';
 MESSAGE47 = 'Print this pa<g>e           Ctrl-p';
 MESSAGE48 = '<Q>uit                      Ctrl-q';
 MESSAGE49 = '<Enter> load';
 {...}
 MESSAGE59 = '   No such component!';
 MESSAGE60 = '       No component!';
 MESSAGE61 = 'Choice';
 MESSAGE62 = '- <P>revious page -';
 MESSAGE63 = '- <N>ext page -';

implementation

// -- beep on error, if it enabled --------------------------------------------
procedure es;
begin
  if esnd then errsound;
end ;

// -- Selector box ------------------------------------------------------------
function fselect(x1,y1 :byte; menuitems: fitems): string;
var
  lastitem: integer;                                   {Number of the last item}
  mp: byte;                                                      {Menu position}
  firstline: integer;                    {Number of the label in the first line}

procedure highlighted;
begin
  gotoxy(1,mp);
  textbackground(button_background);
  textcolor(button_foreground);
  write(' '+menuitems[firstline+mp-1]);
  for b:=length(menuitems[firstline+mp-1]) to 18 do write(' ');
  textbackground(menu_background);
  textcolor(menu_foreground);
end;

procedure writeitems(redraw: boolean);
begin
  if redraw=true then
  begin
    window(x1+2,y1+1,x1+25,y1+12);
    textbackground(black);
    clrscr;
    textbackground(menu_background);
    textcolor(menu_foreground);
    win(x1,y1,x1+22+1,y1+11);
  end
  else
    begin
      window(x1+2,y1+1,x1+21,y1+10);
      clrscr;
      window(x1+2,y1+1,x1+21,y1+11);
      for i:=firstline to firstline+9 do writeln(' '+menuitems[i]);
    end;
end;

begin
  if menuitems[1]='' then
  begin
    fselect:='@';
    exit;
  end;
  for i:=1 to 6000 do
    if menuitems[i]<>'' then lastitem:=i;
  firstline:=1;
  c:='a';
  mp:=1;
  repeat
    writeitems(true);
    repeat
      case c of
         #72: if mp>1 then mp:=mp-1                                         {Up}
                else if firstline>1 then firstline:=firstline-1;
         #80: if (mp<10) and (lastitem>mp) then mp:=mp+1                  {Down}
                else if lastitem>firstline+9 then firstline:=firstline+1;
        #119: begin                                                  {Ctrl-Home}
                mp:=1;
                firstline:=1;
              end;
         #71: mp:=1;                                                      {Home}
         #79: if lastitem<10 then mp:=lastitem else                        {End}
                if lastitem>firstline+9 then mp:=10 else
                begin
                  firstline:=lastitem-9;
                  mp:=10;
                end;
        #117: if lastitem>10 then                                     {Ctrl-End}
              begin
                firstline:=lastitem-9;
                mp:=10;
              end else mp:=lastitem;
         #73: if firstline>10 then firstline:=firstline-10 else        {Page Up}
              begin
                firstline:=1;
                mp:=1;
              end;
         #81: if lastitem>10 then                                    {Page Down}
                if lastitem>firstline+20 then firstline:=firstline+10 else
                begin
                  firstline:=lastitem-9;
                  mp:=10;
                end;
      end;
      writeitems(false);
      highlighted;
      c:=readkey;
      if c=#0 then c:=readkey;
    until (c=#13) or (c=#27);
    if c=#13 then
    begin
      fselect:=menuitems[firstline+mp-1];
      exit;
    end;
  until c=#27;
  fselect:='?';
end;

// -- Make frame for viewer ----------------------------------------------------
procedure frame;
begin
  window(1,1,80,25); textbackground(base_background); clrscr;
  header(headtext);
  window(1,1,80,25);
  footer(MESSAGE05);
  textbackground(base_background);
  textcolor(base_foreground);
  win(1,2,80,24);
  window(1,2,80,25);
  gotoxy(2,3); write(#195);
  for b:=2 to 48 do write(#196);
  write(#194);
  for b:=51 to 78 do write(#196);
  write(#180);
  gotoxy(19,1); write(#194);
  gotoxy(19,2); write(#179);
  gotoxy(19,3); write(#193);
  for b:=4 to 22 do
  begin
    gotoxy(50,b); write(#179);
  end;
  gotoxy(50,23); write(#193);
  gotoxy(50,5); write(#195);
  for b:=51 to 78 do write(#196);
  write(#180);
  window(1,1,80,25);
end;

// -- Load datafile ------------------------------------------------------------
procedure loadedf;
var
  x: string;
  recs: string;
  ml, cl, e: integer;
  textfile: text;
  loaderr: boolean;
  
begin
  settextbuf(textfile,dfb);
  assign(textfile,exepath+'..\library\ver_'+lng+'.txt');
  {$I-}
    reset(textfile);
  {$I+}
  if ioresult=0 then
  begin
    readln(textfile,s);
    datafileversion:=rmchr3(s);
    closefile(textfile);
  end;
  assign(textfile,exepath+'..\library\tubes_'+lng+'.edf');
  {$I-}
    reset(textfile);
  {$I+}
  if ioresult=0 then
  begin
    for b:=1 to 64 do ctgr^[b]:='';
    for i:=1 to 6000 do cpnm^[i]:='';
    for i:=1 to 6000 do cpct^[i]:=0;
    footer('');
    gauge(true,MESSAGE02,0);
    recs:=''; cl:=0; ml:=0;
    repeat
      readln(textfile,s);
      s:=rmchr4(s);
      x:=s; delete(x,5,length(x));
      if x='RECS' then
      begin
        for b:=5 to length(s) do recs:=recs+s[b];
        val(recs,ml,e);
      end;
      if (x='DSCR') then for b:=5 to length(s) do dscr:=dscr+s[b];
      if (x='CTGR') then
      begin
        b:=1;
        iii:=1;
        i:=1;
        repeat
          readln(textfile,s);
          s:=rmchr4(s);
          if s<> 'END' then ctgr^[b]:=s;
          b:=b+1;
        until (s='end') or (b=65);
      end;
      if (x='CPNM') then
        if i<8001 then
        begin
          s:=rmchr1(s);
          for b:=5 to length(s) do cpnm^[i]:=cpnm^[i]+s[b];
          i:=i+1;
          cl:=cl+1;
          if e<>trunc(cl/(ml/100)) then
            gauge(false,'',trunc(cl/(ml/100)));
          e:=trunc(cl/(ml/100));
        end;
      if (x='CPCT') then
        if i<8001 then
        begin
          s:=rmchr1(s);
          delete(s,1,4);
          val(s,cpct^[iii],ii);
          iii:=iii+1;
        end;
    until eof(textfile);
    close(textfile);
    loaderr:=not loaddatafile;
    gauge(false,'',100);
    a_cpnm:=cpnm^[1];
    a_cpct:=ctgr^[1];
    ldp:=1;
    pp:=1;
    for lastcomp:=1 to 6000 do
      if length(cpnm^[lastcomp])=0 then break;
    lastcomp:=lastcomp-1;
  end else loaderr:=true;
  if loaderr then
  begin
    with messtext do
    begin
      title:=message07;
      message:=message04;
      ok:=' Ok ';
      cancel:='';
      foot_message:=message03;
    end;
    es;
    error_box(messtext);
    quit(3,'');
  end;
  frame;
end;

// -- Catalog page loader ------------------------------------------------------
procedure pageload(name: string);
var
  smp: string;
  gco: byte;
  cpnum: integer;
  textfile: text;
  found: byte;
  bx, x1, x2, y1, y2: byte;
label
  next_read;

begin
  settextbuf(textfile,dfb);
  assign(textfile,exepath+'..\library\tubes_'+lng+'.edf');
  {$I-}
    reset(textfile);
  {$I+}
  if ioresult=0 then
  begin
    gco:=0;
    cpnum:=0;
    found:=0;
    for i:=1 to 6000 do
      if cpnm^[i]=name then
      begin
        cpnum:=cpnum+1;
      end;
    if cpnum=0 then
    begin
      with messtext do
      begin
        title:=MESSAGE09;
        message:=MESSAGE59;
        ok:=' OK ';
        cancel:=MESSAGE10;
        foot_message:=MESSAGE11;
      end;
      message_box(messtext);
      exit;
    end;
   next_read:
    gauge2(true,MESSAGE38);
    for b:=1 to 48 do a_cppn^[b]:='';
    for b:=1 to 90 do a_cpld^[b]:='';
    a_cpds:='';
    a_cppc:='';
    a_cpnm:=name;
    repeat
      readln(textfile,s);
      s:=rmchr4(s);
      s:=rmchr1(s);
      smp:=s; delete(smp,5,length(smp));
      if smp='CPNM' then gco:=gco+1;
      if gco=10 then
      begin
        gauge2(false,'');
        gco:=0
      end;
    until s='CPNM'+name;
    found:=found+1;
    repeat
      readln(textfile,s);
      s:=rmchr4(s);
      s:=rmchr3(s);
      if (s[1]+s[2]+s[3]+s[4]='CPDS') then
        for b:=6 to length(s) do a_cpds:=a_cpds+s[b];
      if (s[1]+s[2]+s[3]+s[4]='CPPC') then
        for b:=6 to length(s) do a_cppc:=a_cppc+s[b];
    until rmchr1(s)='CPPN';
    for b:=1 to 48 do
    begin
      readln(textfile,s);
      s:=rmchr4(s);
      s:=rmchr3(s);
      if rmchr1(s)<>'END' then a_cppn^[b]:=s else break;
    end;
    repeat
      readln(textfile,s);
      s:=rmchr4(s);
    until rmchr1(s)='CPLD';
    for b:=1 to 90 do
    begin
      readln(textfile,s);
      s:=rmchr4(s);
      s:=rmchr3(s);
      if rmchr1(s)<>'END' then a_cpld^[b]:=s else break;
    end;
    // more result
    if cpnum>1 then
    begin
      x1:=17; y1:=7; x2:=67; y2:=20;
      window(x1,y1,x2,y2); textbackground(black); clrscr;
      textbackground(colors[4]); textcolor(colors[3]);
      win(x1-2,y1-1,x2-2,y2-1);
      gotoxy(3,1); write(' ',MESSAGE41,' ');
      window(x1+1,y1,x2-5,y2-4);
      writeln(MESSAGE42+': ',found,'/',cpnum);
      writeln;
      writeln('- ' + a_cpnm);
      writeln('- ' + a_cpds);
      writeln('- ' + a_cpld^[1]);
      writeln('  ' + a_cpld^[2]);
      writeln('  ' + a_cpld^[3]);
      writeln('  ' + a_cpld^[4]);
      writeln('  ' + a_cpld^[5]);
      write('  (...)');
      line(x1,x2-2,y1+1);
      line(x1,x2-2,y1+10);
      if found=cpnum then
      begin
        window(1,1,80,25);
        textbackground(colors[12]); textcolor(colors[10]);
        gotoxy(x1+5,y1+11); writeln('[ '+MESSAGE43+' ]');
        footer(MESSAGE49);
        repeat
         c:=readkey;
        until c=#13;
      end else
      begin
        footer(MESSAGE40);
        window(1,1,80,25);
        textbackground(colors[4]); textcolor(colors[3]);
        gotoxy(x1+24,y1+11); write('[ '+MESSAGE44+' ]');
        textbackground(colors[12]); textcolor(colors[10]);
        gotoxy(x1+13,y1+11); write('[ '+MESSAGE43+' ]');
        window(x1+1,y1+1,x2-5,y2-4);
        bx:=0;
        repeat
          c:=readkey;
          if (c=#9) and (bx=0) then
          begin
            window(1,1,80,25);
            textbackground(colors[4]); textcolor(colors[3]);
            gotoxy(x1+13,y1+11); write('[ '+MESSAGE43+' ]');
            textbackground(colors[12]); textcolor(colors[10]);
            gotoxy(x1+24,y1+11); write('[ '+MESSAGE44+' ]');
            window(x1+1,y1+1,x2-5,y2-4);
            bx:=1;
            c:=#0;
          end;
          if (c=#9) and (bx=1) then
          begin
            window(1,1,80,25);
            textbackground(colors[4]); textcolor(colors[3]);
            gotoxy(x1+24,y1+11); write('[ '+MESSAGE44+' ]');
            textbackground(colors[12]); textcolor(colors[10]);
            gotoxy(x1+13,y1+11); write('[ '+MESSAGE43+' ]');
            window(x1+1,y1+1,x2-5,y2-4);
            bx:=0;
            c:=#0;
          end;
        until (c=#27) or (c=#13);
        if (c=#27) or (bx=1) then goto next_read;
      end;
    end;
    close(textfile);
    for b:=1 to 90 do
      if a_cpld^[b]<>'' then ldend:=b;
    for b:=1 to 48 do
      if a_cppn^[b]<>'' then pend:=b;
    ldpmax:=1;
    if ldend > 19 then ldpmax:=2;
    if ldend > 38 then ldpmax:=3;
    if ldend > 57 then ldpmax:=4;
    if ldend > 76 then ldpmax:=5;
    ppmax:=1;
    if pend > 17 then ppmax:=2;
    if pend > 34 then ppmax:=3;
  end else
  begin
    with messtext do
    begin
      title:=MESSAGE09;
      message:=MESSAGE12;
      ok:=' OK ';
      cancel:=MESSAGE10;
      foot_message:=MESSAGE11;
    end;
    message_box(messtext);
  end;
end;

// -- Save actual page to text file --------------------------------------------
procedure pagesave;
var
  fname,bfname: string;
  textfile: text;

procedure io_error;
begin
  with messtext do
  begin
    title:=MESSAGE07;
    message:=MESSAGE13;
    ok:=' OK ';
    cancel:='';
    foot_message:=MESSAGE03;
  end;
  es;
  message_box(messtext);
end;

begin
  with messtext do
  begin
    title:=MESSAGE14;
    message:=MESSAGE15;
    ok:=' OK ';
    cancel:=MESSAGE10;
    foot_message:=MESSAGE11;
  end;
  fname:=inputbox(8,messtext)+'.txt';
  if fname='?.txt' Then exit;
  if fsearch(fname,'')<>'' Then
  begin
    bfname:=fname;
    delete(bfname,length(bfname)-2,3);
    assign(textfile,fname);
    bfname:=bfname+'.bak';
    {$I-}
    rename(textfile,bfname);
    {$I+}
    if ioresult<>0 then
    begin
      io_error;
      exit;
    end;
  end;
  {$I-}
  assign(textfile,fname);
  rewrite(textfile);
  {$I+}
  if ioresult<>0 then
  begin
    io_error;
    exit;
  end;
  writeln(textfile,MESSAGE16+' '+a_cpnm);
  writeln(textfile,MESSAGE17+' '+a_cpds);
  writeln(textfile,MESSAGE18+' '+a_cppc);
  writeln(textfile,'');
  writeln(textfile,MESSAGE19);
  for b:=1 to 48 do
    if length(a_cppn^[b])>0 then
    begin
      if b<=9 then write(textfile,' ',b,': ') else write(textfile,b,': ');
      writeln(textfile,a_cppn^[b]);
    end;
  writeln(textfile,'');
  writeln(textfile,MESSAGE20);
  for b:=1 to 90 do
    if (length(a_cpld^[b])<>0) or (length(a_cpld^[b+1])<>0)
      then writeln(textfile,a_cpld^[b]);
  close(textfile);
end;

// -- Print actual page --------------------------------------------------------
procedure pageprint;

procedure io_error;
begin
  with messtext do
  begin
    title:=MESSAGE07;
    message:=MESSAGE21;
    ok:=' OK ';
    cancel:='';
    foot_message:=MESSAGE03;
  end;
  es;
  message_box(messtext);
end;

begin
  {$I-}
  writeln(LST,MESSAGE16+' '+a_cpnm);
  writeln(LST,MESSAGE17+' '+a_cpds);
  writeln(LST,MESSAGE18+' '+a_cppc);
  writeln(LST,'');
  writeln(LST,MESSAGE19);
  for b:=1 to 48 do
    if length(a_cppn^[b])>0 then
    begin
      if b<=9 then write(LST,' ',b,': ') else write(LST,b,': ');
      writeln(LST,a_cppn^[b]);
    end;
  writeln(LST,'');
  writeln(LST,MESSAGE20);
  for b:=1 to 90 do
    if (length(a_cpld^[b])<>0) or (length(a_cpld^[b+1])<>0)
      then writeln(LST,a_cpld^[b]);
  if pgdr=true then writeln(lst,#12);
  {$I+}
  if ioresult<>0 then
  begin
    io_error;
    exit;
  end;
end;

// -- View actual page ---------------------------------------------------------
procedure pageview(ld_page, p_page: byte);
begin
  textbackground(base_background);
  textcolor(base_foreground);
  window(1,1,80,25);
  gotoxy(4,24);  write('[ < ]-[ > ]');
  gotoxy(52,24); write('[ < ]-[ > ]');
  gotoxy(44,24); write('[',ld_page,'/',ldpmax,']');
  gotoxy(73,24); write('[',p_page,'/',ppmax,']');
  textcolor(base_foreground-8);
  if ld_page=1 then
  begin
    gotoxy(6,24); write('<');
  end;
  if ld_page=ldpmax then
  begin
    gotoxy(12,24); write('>');
  end;
  if p_page=1 then
  begin
    gotoxy(54,24); write('<');
  end;
  if p_page=ppmax then
  begin
    gotoxy(60,24); write('>');
  end;
  textcolor(base_foreground);
  window(4,3,16,3); clrscr;
  write(a_cpnm);
  window(21,3,77,3); clrscr;
  write(a_cpds);
  window(52,5,77,5); clrscr;
  write(a_cppc);
  window(4,5,48,23); clrscr;
  window(4,5,48,24);
  for b:=(ld_page*19)-18 to ld_page*19 do
    writeln(a_cpld^[b]);
  window(52,7,77,23); clrscr;
  window(52,7,79,24);
  for b:=(p_page*17)-16 to p_page*17 do
  begin
    if length(a_cppn^[b])>0 then
      if b<=9 then write(' ',b,': ') else write(b,': ');
      writeln(a_cppn^[b]);
  end;
end;

// -- Search component ---------------------------------------------------------
procedure search_menu;
begin
  with messtext do
  begin
    title:=MESSAGE09;
    message:=MESSAGE16;
    ok:=' OK ';
    cancel:=MESSAGE10;
    foot_message:=MESSAGE11;
  end;
  s:=inputbox(16,messtext);
  for b:=1 to length(s) do s[b]:=upcase(s[b]);
  if s<>'?' then
  begin
    frame;
    pageview(ldp,pp);
    pageload(s);
  end;
end;

// -- Choice from menu ---------------------------------------------------------
procedure choice_menu;
var
  fselectitems: fitems;
  citems: itemrec;
label
  p1, p2, p3, p4, p5;
begin
  p1:
  frame;
  pageview(ldp,pp);

  with citems do
  begin
    i1:=ctgr^[1];
    i2:=ctgr^[2];
    i3:=ctgr^[3];
    i4:=ctgr^[4];
    i5:=ctgr^[5];
    i6:=ctgr^[6];
    i7:=ctgr^[7];
    i8:=ctgr^[8];
    i9:=ctgr^[9];
    i10:=ctgr^[10];
    i11:=ctgr^[11];
    i12:=ctgr^[12];
    i13:=ctgr^[13];
    i14:=ctgr^[14];
    i15:=ctgr^[15];
    i16:=ctgr^[16];
    i17:=#175;
    i18:=''; i19:=''; i20:='';
  end;
  footer(MESSAGE22);
  b:=menu(3,3,citems);
  if b=255 then exit;
  if b=17 then goto p2 else goto p5;

  p2:
  frame;
  pageview(ldp,pp);
  with citems do
  begin
    i1:=#174;
    i2:=ctgr^[17];
    i3:=ctgr^[18];
    i4:=ctgr^[19];
    i5:=ctgr^[20];
    i6:=ctgr^[21];
    i7:=ctgr^[22];
    i8:=ctgr^[23];
    i9:=ctgr^[24];
    i10:=ctgr^[25];
    i11:=ctgr^[26];
    i12:=ctgr^[27];
    i13:=ctgr^[28];
    i14:=ctgr^[29];
    i15:=ctgr^[30];
    i16:=ctgr^[31];
    i17:=ctgr^[32];
    i18:=#175;
    i19:=''; i20:='';
  end;
  footer(MESSAGE22);
  b:=menu(3,3,citems);
  if b=255 then exit;
  if b=1 then goto p1;
  if b=18 then goto p3 else
  begin
    b:=16+(b-1);
    goto p5;
  end;

  p3:
  frame;
  pageview(ldp,pp);
  with citems do
  begin
    i1:=#174;
    i2:=ctgr^[33];
    i3:=ctgr^[34];
    i4:=ctgr^[35];
    i5:=ctgr^[36];
    i6:=ctgr^[37];
    i7:=ctgr^[38];
    i8:=ctgr^[39];
    i9:=ctgr^[40];
    i10:=ctgr^[41];
    i11:=ctgr^[42];
    i12:=ctgr^[43];
    i13:=ctgr^[44];
    i14:=ctgr^[45];
    i15:=ctgr^[46];
    i16:=ctgr^[47];
    i17:=ctgr^[48];
    i18:=#175;
    i19:=''; i20:='';
  end;
  footer(MESSAGE22);
  b:=menu(3,3,citems);
  if b=255 then exit;
  if b=1 then goto p2;
  if b=18 then goto p4 else
  begin
    b:=32+(b-1);
    goto p5;
  end;

  p4:
  frame;
  pageview(ldp,pp);
  with citems do
  begin
    i1:=#175;
    i2:=ctgr^[49];
    i3:=ctgr^[50];
    i4:=ctgr^[51];
    i5:=ctgr^[52];
    i6:=ctgr^[53];
    i7:=ctgr^[54];
    i8:=ctgr^[55];
    i9:=ctgr^[56];
    i10:=ctgr^[57];
    i11:=ctgr^[58];
    i12:=ctgr^[59];
    i13:=ctgr^[60];
    i14:=ctgr^[61];
    i15:=ctgr^[62];
    i16:=ctgr^[63];
    i17:=ctgr^[64];
    i18:=''; i19:=''; i20:='';
  end;
  footer(MESSAGE22);
  b:=menu(3,3,citems);
  if b=255 then exit;
  if b=1 then goto p3;
  b:=48+(b-1);

  p5:
  for i:=1 to 6000 do fselectitems[i]:='';
  ii:=1;
  for i:=1 to 6000 do
  begin
    if cpct^[i]=b then
    begin
      if (i=1) then
      begin
        fselectitems[ii]:=cpnm^[i];
        ii:=ii+1;
      end else
        if (cpnm^[i]<>cpnm^[i-1]) then
        begin
          fselectitems[ii]:=cpnm^[i];
          ii:=ii+1;
        end;
    end;
  end;
  footer(MESSAGE23);
  s:=fselect(40,11,fselectitems);
  if (s<>'?') and (s<>'@') then
  begin
    frame;
    pageview(ldp,pp);
    pageload(s);
  end;
  if s='@' then
  begin
    with messtext do
    begin
      title:=MESSAGE61;
      message:=MESSAGE60;
      ok:=' OK ';
      cancel:='';
      foot_message:=MESSAGE03;
    end;
    frame;
    pageview(ldp,pp);
    message_box(messtext);
  end;
end;

// -- View tube base with external viewer --------------------------------------
procedure viewtubebase;
var
 picsfile: string[12];
begin
  window(1,1,80,25);
  textcolor(lightgray); textbackground(black);
  clrscr;
  picsfile:=a_cppc;
  if length(picsfile)>8 then delete(picsfile,8,length(picsfile));
  picsfile:=picsfile+'.png';
  swapvectors;
  exec(picviewer,exepath+'..\library\base\'+picsfile);
  swapvectors;
  if doserror<>0 then
  begin
    with messtext do
    begin
      title:=message07;
      message:=message26;
      ok:=' Ok ';
      cancel:='';
      foot_message:=message03;
    end;
    es;
    error_box(messtext);
  end;
end;

// -- Catalog pager viewer -----------------------------------------------------
procedure catalog_pages;
var
  items: itemrec;
label
  menulabel;
begin
  new(cpct);
  new(cpnm);
  new(ctgr);
  new(a_cpld);
  new(a_cppn);
  headtext:=ucommon.PRGNAME+' v'+ucommon.PRGVER+' - '+MESSAGE06;
  with items do
  begin
    i1:=MESSAGE32;
    i2:=MESSAGE33;
    i3:=MESSAGE25;
    i4:=MESSAGE34;
    i5:='-';
    i6:=MESSAGE39;
    i7:=MESSAGE45;
    i8:='-';
    i9:=MESSAGE46;
    i10:=MESSAGE47;
    i11:='-';
    i12:=MESSAGE35;
    i13:=MESSAGE48;
    i14:=''; i15:=''; i16:=''; i17:=''; i18:=''; i19:=''; i20:='';
  end;
  background;
  header(headtext);
  loadedf;
  pageload(a_cpnm);
  menulabel:
  frame;
  pageview(ldp,pp);
  repeat
    showtime(true);
  until keypressed;
  showtime(false);
  c:=readkey;
  if c=#0 then c:=readkey;
  if c=#68 then                                                            {F10}
  begin
    footer(MESSAGE22);
    case menu(3,2,items) of
      1: c:=#60;
      2: c:=#61;
      3: c:=#62;
      4: substcontent;
      6: helpbox('p');
      7: helpcontent;
      9: pagesave;
      10: pageprint;
      12: if dosshell(MESSAGE36)<>0 then
          begin
            writeln(MESSAGE37);
            readkey;
          end;
      13: exit;
    end;
  end;
  case c of
    #19: pagesave;                                                      {Ctrl-s}
    #16: pageprint;                                                     {Ctrl-p}
    #17: exit;                                                          {Ctrl-q}
    #59: helpbox('3');                                                      {F1}
    #60: choice_menu;                                                       {F2}
    #61: search_menu;                                                       {F3}
    #62: viewtubebase;                                                      {F4}
    #71: ldp:=1;                                                          {Home}
    #79: ldp:=ldpmax;                                                      {End}
    #80: If ldp*19<ldend Then ldp:=ldp+1;                                 {Down}
    #72: ldp:=ldp-1;                                                        {Up}
    #77: If pp*17<pend Then pp:=pp+1;                                    {Right}
    #75: pp:=pp-1;                                                        {Left}
  end;
  if ldp=6 then ldp:=5;
  if ldp=0 then ldp:=1;
  if pp=4 then pp:=3;
  if pp=0 then pp:=1;
  goto menulabel;
  dispose(cpct);
  dispose(cpnm);
  dispose(ctgr);
  dispose(a_cpld);
  dispose(a_cppn);
end;

End.
