{ +--------------------------------------------------------------------------+ }
{ | Tubes2 for DOS 0.2 Open edition * Electron tube catalog                  | }
{ | Copyright (C) 2016-2017 Pozsar Zsolt <info@pozsarzs.hu>                  | }
{ | ucommon.pas                                                              | }
{ | common procedures and functions                                          | }
{ +--------------------------------------------------------------------------+ }

{
   This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by the
 Free Software Foundation, either version 3 of the License, or (at your
 option) any later version.

   This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

   You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit ucommon;
interface
uses
  dos, crt;
type
  itemrec=record
            i1, i2, i3, i4, i5, i6, i7, i8, i9, i10,
            i11, i12, i13, i14, i15, i16, i17, i18, i19, i20: string[50];
          end;
  fitems=array[1..6000] of string[16];
  messagerec=record
               title, message, ok, cancel, foot_message: string;
             end;
var
  b, bb: byte;
  c: char;
  colors: array[1..12] of byte;
  gpos: byte;
  s, ss: string;
  textfile: text;
const
  PRGNAME='Tubes2 for DOS Open edition';
  PRGVER='0.2';

{
 Colours:
 1: base foreground,                    7: header, footer, menu foreground,
 2: base background,                    8: header, footer, menu highlighted t.,
 3: message box foreground,             9: header, footer, menu background,
 4: message box background,             10: field-cursor foreground,
 5: error message box foreground,       11: field-cursor highlighted text,
 6: error message box background,       12: field-cursor background.
}

function base_background: byte;
function base_foreground: byte;
function button_background: byte;
function button_foreground: byte;
function button_highlighted: byte;
function error_background: byte;
function error_foreground: byte;
function inputbox(tl: byte; messages: messagerec): string;
function loadpalfile(filename: string): boolean;
function menu(x1,y1: byte; menuitems: itemrec): byte;
function menu_background: byte;
function menu_foreground: byte;
function menu_highlighted: byte;
function message_background: byte;
function message_foreground: byte;
function rmchr1(input: string): string;
function rmchr2(input: string): string;
function rmchr3(input: string): string;
function rmchr4(input: string): string;
procedure background;
function dosshell(message: string): integer;
procedure error_box(messages: messagerec);
procedure ewrite(fg_color: byte; hl_color: byte; vtext: string);
procedure footer(title: string);
procedure gauge(ff: boolean; title: string; percent: byte);
procedure gauge2(ff: boolean; title: string);
procedure header(title: string);
procedure line(x1, x2, y: byte);
procedure message_box(messages: messagerec);
procedure quit(halt_code: byte; message: string);
procedure win(x1,y1,x2,y2: byte);
procedure showtime(show: boolean);
procedure cursoron;
procedure cursoroff;
procedure errsound;

implementation
uses
  ucatalog;

// -- make error sound -------------------------------------------------------
procedure errsound;
begin
  sound(3000); delay(100);
  sound(2000); delay(100);
  nosound;
end;

// -- show time in header -----------------------------------------------------
procedure showtime(show: boolean);
var
  o,p,mp,szmp: word;

function ido(v: word): string;
var
  u: string;
begin
  str(v:0,u);
  if length(u)=1 then u:='0'+u;
  ido:=u;
end;

begin
  window(1,1,80,1);
  textcolor(colors[7]); textbackground(colors[9]);
  if show then
  begin
    getTime(o,p,mp,szmp);
    gotoxy(72,1); write(ido(o)+':'+ido(p)+':'+ido(mp));
  end else
  begin
    gotoxy(72,1); write('        ');
  end;
end;

// -- show cursor on text mode ------------------------------------------------
procedure cursoron;
var
 r: registers;
begin
  r.AL:=$00; r.AH:=$01;
  r.CL:=$07; r.CH:=$06;
  intr($10,r);
end;

// -- hide cursor on text mode ------------------------------------------------
procedure cursoroff;
var
 r: registers;
begin
  r.AL:=$00; r.AH:=$01;
  r.CL:=$00; r.CH:=$10;
  intr($10,r);
end;

// -- make solid background ----------------------------------------------------
procedure background;
const
 lll='';
begin
  window(1,1,80,25);
  textbackground(colors[1]);textcolor(colors[2]);
  for b:=2 to 24 do
  begin
    gotoxy(1,b);
    write(lll);
  end;
end;

// -- write procedure with highlighted words -----------------------------------
procedure ewrite(fg_color: byte; hl_color: byte; vtext: string);
var
  b: byte;
begin
  textcolor(fg_color);
  for b:=1 to length(vtext) do
  begin
    case vtext[b] of
      '<': textcolor(hl_color);
      '>': textcolor(fg_color);
    else write(vtext[b]);
    end;
  end;
end;

// -- footer -------------------------------------------------------------------
procedure footer(title: string);
begin
  window(1,25,80,25);
  textcolor(colors[7]); textbackground(colors[9]); clreol;
  ewrite(colors[7],colors[8],' '+title);
  gotoxy(71,1); write(ucatalog.datafileversion);
end;

// -- header -------------------------------------------------------------------
procedure header(title: string);
begin
  window(1,1,80,1);
  textcolor(colors[7]); textbackground(colors[9]); clreol;
  ewrite(colors[7],colors[8],' '+title);
end;

// -- gauge ------------------------------------------------------------------
procedure gauge(ff: boolean; title: string; percent: byte);
var
  x1, y1: byte;
  x2, y2: byte;
begin
  x1:=26;
  y1:=11;
  x2:=x1+30;
  y2:=y1+3;
  if percent>100 then percent:=100;
  if ff=true then
  begin
    window(x1,y1,x2,y2);
    textbackground(black); clrscr;
    textbackground(colors[4]); textcolor(colors[3]);
    win(x1-2,y1-1,x2-2,y2-1);
    gotoxy(3,1); write(' ',title,' ');
    window(x1+1,y1+1,x2-2,y2-1);
    gotoxy(1,1); for b:=1 to 25 do write(#177);
  end;
  textbackground(colors[4]); textcolor(colors[3]);
  window(x1+1,y1+1,x2-2,y2-1);
  gotoxy(1,1); for b:=1 to percent div 4 do write(#219);
end;

// -- gauge2 ------------------------------------------------------------------
procedure gauge2(ff: boolean; title: string);
var
  x1, y1: byte;
  x2, y2: byte;
begin
  x1:=26;
  y1:=11;
  x2:=x1+30;
  y2:=y1+3;
  if ff=true then
  begin
    window(x1,y1,x2,y2);
    textbackground(black); clrscr;
    textbackground(colors[4]); textcolor(colors[3]);
    win(x1-2,y1-1,x2-2,y2-1);
    gotoxy(3,1); write(' ',title,' ');
    window(x1+1,y1+1,x2-2,y2-1);
    gotoxy(1,1); for b:=1 to 25 do write(#177);
    gpos:=0;
  end;
  textbackground(colors[4]); textcolor(colors[3]);
  window(x1+1,y1+1,x2-2,y2-1);
  gpos:=gpos+1;
  if gpos=26 then gpos:=1;
  if gpos=1 then
  begin
    gotoxy(25,1); write(#177);
    gotoxy(gpos,1); write(#219);
  end else
  begin
    gotoxy(gpos,1);write(#219);
    gotoxy(gpos-1,1); write(#177);
  end;
end;

// -- framed window ------------------------------------------------------------
procedure win(x1,y1,x2,y2: byte);
begin
  window(x1,y1,x2,y2);
  clrscr;
  window(x1,y1,x2,y2+1);
  write(' '+#218);
  for b:=1 to (x2-x1-3) do write(#196);
  writeln(#191);
  for b:=1 to (y2-y1-1) do writeln(' '+#179);
  for b:=1 to (y2-y1-1) do
  begin
    gotoxy(x2-x1,b+1); writeln(#179);
  end;
  gotoxy(1,y2-y1+1); write(' '+#192);
  for b:=1 to (x2-x1-3) do write(#196);
  write(#217);
end;

// -- message box --------------------------------------------------------------
procedure message_box(messages: messagerec);
var
  x1, y1: byte;
  x2, y2: byte;

begin
  x1:=26;
  y1:=10;
  x2:=x1+30;
  y2:=y1+5;
  footer(messages.foot_message);
  window(x1,y1,x2,y2);
  textbackground(black); clrscr;
  textbackground(colors[4]); textcolor(colors[3]);
  win(x1-2,y1-1,x2-2,y2-1);
  gotoxy(3,1); write(' ',messages.title,' ');
  window(x1+1,y1,x2-5,y2-4);
  write(messages.message);
  window(1,1,80,25);
  textbackground(colors[12]); textcolor(colors[10]);
  gotoxy(x1+11,y1+3); writeln('[ '+messages.ok+' ]');
  repeat
    c := readkey;
  until (c=#27) or (c=#13); {Escape, Enter}
  c := ' ';
end;

// -- error message box --------------------------------------------------------
procedure error_box(messages: messagerec);
var
  x1, y1: byte;
  x2, y2: byte;

begin
  x1:=26;
  y1:=10;
  x2:=x1+30;
  y2:=y1+5;
  footer(messages.foot_message);
  window(x1,y1,x2,y2);
  textbackground(black); clrscr;
  textbackground(colors[6]); textcolor(colors[5]);
  win(x1-2,y1-1,x2-2,y2-1);
  gotoxy(3,1); write(' ',messages.title,' ');
  window(x1+1,y1,x2-5,y2-4);
  write(messages.message);
  window(1,1,80,25);
  textbackground(colors[12]); textcolor(colors[10]);
  gotoxy(x1+11,y1+3); writeln('[ '+messages.ok+' ]');
  repeat
    c := readkey;
  until (c=#27) or (c=#13); {Escape, Enter}
  c := ' ';
end;

// -- make a horisontal line ---------------------------------------------------
procedure line(x1, x2, y: byte);
begin
  window(1,1,80,25);
  gotoxy(x1,y);
  repeat
    write(#196);
  until wherex=x2-1;
end;

// -- menu box -----------------------------------------------------------------
function menu(x1,y1: byte; menuitems: itemrec): byte;
var
  items: array[1..20] of string[50];
  hotkeys: array[1..20] of char;
  mp, maxmp: byte;
  x2, y2: byte;

procedure writeitems(redraw: boolean);
begin
  if redraw=true then
  begin
    window(x1+2,y1+1,x2+3,y2+1);
    textbackground(black); clrscr;
    textbackground(colors[9]); textcolor(colors[7]);
    win(x1,y1,x2+1,y2);
  end;
  for bb:=1 to maxmp do
  begin
    window(x1,y1+1,x2+1,y2);
    textbackground(colors[9]);
    if items[bb]='-' then
    begin
      textcolor(colors[7]); line(x1+2,x2+1,bb+y1);
    end else
    begin
      window(x1,y1+1,x2-1,y2); gotoxy(3,bb); clreol;
      window(x1,y1+1,x2+1,y2); gotoxy(3,bb); ewrite(colors[7],colors[8],' '+items[bb]);
    end;
  end;
end;

procedure highlighted;
begin
  textbackground(colors[12]);
  window(x1,y1+1,x2-1,y2); gotoxy(3,mp); clreol;
  window(x1,y1+1,x2+1,y2); gotoxy(3,mp);
  ewrite(colors[10],colors[11],' '+items[mp]);
end;

begin
  for b:=1 to 20 do items[b] := '';
  x2 := 0; y2 := 0;
  with menuitems do
  begin
    items[1] := i1;
    items[2] := i2;
    items[3] := i3;
    items[4] := i4;
    items[5] := i5;
    items[6] := i6;
    items[7] := i7;
    items[8] := i8;
    items[9] := i9;
    items[10] := i10;
    items[11] := i11;
    items[12] := i12;
    items[13] := i13;
    items[14] := i14;
    items[15] := i15;
    items[16] := i16;
    items[17] := i17;
    items[18] := i18;
    items[19] := i19;
    items[20] := i20;
  end;
  for b:=1 to 20 do
    for bb:=1 to length(items[b]) do
      if items[b,bb]='<' then hotkeys[b] := lowercase(items[b,bb+1]);
  b := 1;
  x2 := length(items[b]);
  for b:=2 to 20 do
    if length(items[b])>x2 then x2 := length(items[b]);
  for b:=1 to 20 do
    if length(items[b])>0 then y2 := y2+1;
  maxmp := y2;
  x2 := x1+x2+4; y2 := y1+y2+1;
  mp := 1;
  c := #71;
  repeat
    writeitems(true);
    repeat
      case c of
        #72: mp := mp-1;  {Up}
        #80: mp := mp+1;  {Down}
        #71: mp := 1;     {Home}
        #79: mp := maxmp; {end}
      end;
      for b:=1 to maxmp do
        if c=hotkeys[b] then
        begin
          menu := b;
          exit;
        end;
      if c=#80 then if items[mp]='-' then mp := mp+1;
      if c=#72 then if items[mp]='-' then mp := mp-1;
      if mp=maxmp+1 then mp := maxmp;
      if mp=0 then mp := 1;
      writeitems(false);
      highlighted;
      c := readkey; if c=#0 then c := readkey;
    until (c=#13) or (c=#27);
    if c=#13 then
    begin
      menu := mp;
      exit;
    end;
  until c=#27;
  menu := 255;
end;

// -- input box ----------------------------------------------------------------
function inputbox(tl: byte; messages: messagerec): string;
var
  x1, y1: byte;
  x2, y2: byte;
label back;
begin
  x1:=26;
  y1:=10;
  x2:=x1+30;
  y2:=y1+5;
  footer(messages.foot_message);
  window(x1,y1,x2,y2);
  textbackground(black); clrscr;
  textbackground(colors[4]); textcolor(colors[3]);
  win(x1-2,y1-1,x2-2,y2-1);
  gotoxy(3,1); write(' ',messages.title,' ');
  window(x1+1,y1,x2-5,y2-4);
  write(messages.message);
  window(1,1,80,25);
  gotoxy(x1+14,y1+3); writeln('[ '+messages.cancel+' ]');
  textbackground(colors[12]); textcolor(colors[10]);
  gotoxy(x1+3,y1+3); writeln('[ '+messages.ok+' ]');
  window(x1+1,y1+1,x2-5,y2-4);
  clrscr;
  s := '';
  b := 0;
  cursoron;
  repeat
    back:
    c := readkey; c := upcase(c);
    if ((c>#47) and (c<#58)) or ((c>#64) and (c<#91)) or (c=#45) then
      if length(s)>tl-1 then write('') else
      begin
        s := s+c;
        clrscr;
        write(s);
      end;
    if (c=#8) then
      if length(s)=0 then write('') else
      begin
        delete(s,length(s),1);
        clrscr;
        write(s);
      end;
    if (c=#9) and (b=0) then
    begin
      window(1,1,80,25);
      textbackground(colors[4]); textcolor(colors[3]);
      gotoxy(x1+3,y1+3); writeln('[ '+messages.ok+' ]');
      textbackground(colors[12]); textcolor(colors[10]);
      gotoxy(x1+14,y1+3); writeln('[ '+messages.cancel+' ]');
      window(x1+1,y1+1,x2-5,y2-4);
      gotoxy(length(s)+1,1);
      b := 1;
      c := #0;
    end;
    if (c=#9) and (b=1) then
    begin
      window(1,1,80,25);
      textbackground(colors[4]); textcolor(colors[3]);
      gotoxy(x1+14,y1+3); writeln('[ '+messages.cancel+' ]');
      textbackground(colors[12]); textcolor(colors[10]);
      gotoxy(x1+3,y1+3); writeln('[ '+messages.ok+' ]');
      window(x1+1,y1+1,x2-5,y2-4);
      gotoxy(length(s)+1,1);
      b := 0;
      c := #0;
    end;
  until (c=#13) or (c=#27);
  cursoroff;
  if (c=#13) and (length(s)=0) then inputbox := '?' else
    if (c=#13) and (b=0) then
    begin
      for b:=1 to length(s) do s[b] := lowercase(s[b]);
      inputbox := s
    end
  else inputbox := '?';
  c := ' ';
end;

// -- get color functions ------------------------------------------------------
function base_foreground: byte;
begin
  base_foreground := colors[1];
end;

function base_background: byte;
begin
  base_background := colors[2];
end;

function message_foreground: byte;
begin
  message_foreground := colors[3];
end;

function message_background: byte;
begin
  message_background := colors[4];
end;

function error_foreground: byte;
begin
  error_foreground := colors[5];
end;

function error_background: byte;
begin
  error_background := colors[6];
end;

function menu_foreground: byte;
begin
  menu_foreground := colors[7];
end;

function menu_highlighted: byte;
begin
  menu_highlighted := colors[8];
end;

function menu_background: byte;
begin
  menu_background := colors[9];
end;

function button_foreground: byte;
begin
  button_foreground := colors[10];
end;

function button_highlighted: byte;
begin
  button_highlighted := colors[11];
end;

function button_background: byte;
begin
  button_background := colors[12];
end;

//  -- load palette file -------------------------------------------------------
function loadpalfile(filename: string): boolean;
var
  loaded: array[1..12] of string[12];
const
  names: array[0..15] of string = ('black','blue','green','cyan','red',
                                   'magenta','brown','lightgray',
                                   'darkgray','lightblue','lightgreen',
                                   'lightcyan','lightred','lightmagenta',
                                   'yellow','white');
begin
  for b:=1 to 12 do loaded[b] := '';
  assign(textfile,filename);
 {$I-}
  reset(textfile);
 {$I+}
  if ioresult=0 then
  begin
    ss := '';
    bb := 1;
    repeat
      readln(textfile,s);
      for b:=1 to length(s) do
        if (s[b]<> #32) and (s[b]<> #9) then ss := ss+s[b];
      if (length(ss)>0) and (ss[1]<>'#') then
      begin
        loaded[bb] := ss;
        bb := bb+1;
      end;
      ss := '';
    until (eof(textfile)) or (bb=13);
    close(textfile);
    loadpalfile := true;
  end
  else loadpalfile := false;
  for b:=1 to 12 do
    for bb:=0 to 15 do
      if loaded[b]=names[bb] then colors[b] := bb;
end;

// -- quit ---------------------------------------------------------------------
procedure quit(halt_code: byte; message: string);
begin
  window(1,1,80, 25);
  textcolor(lightgray); textbackground(black); clrscr;
  writeln(message);
  cursoron;
  halt(halt_code);
end;

// -- remove all space and tabulator -------------------------------------------
function rmchr1(input: string): string;
begin
  rmchr1 := '';
  for b:=1 to length(input) do
    if (input[b]<> #32) and (input[b]<> #9)
      then rmchr1 := rmchr1+input[b];
end;

// -- remove all characters after hash -----------------------------------------
function rmchr2(input: string): string;
begin
  rmchr2 := '';
  for b:=1 to length(input) do
    if (input[b]<> #35) then rmchr2 := rmchr2+input[b]
    else break;
end;

// -- remove space and tabulator from start of line ----------------------------
function rmchr3(input: string): string;
begin
  rmchr3 := '';
  while (input[1]=#9) or (input[1]=#32) do delete(input,1,1);
  rmchr3 := input;
end;

// -- it removes bad characters from input string ------------------------------
function rmchr4(input: string): string;
begin
  input := rmchr3(input);
  if input[1]<>'#' then
    begin
      s := '';
      for b:=1 to length(input) do
        if (input[b]<> #35) then s := s+input[b]
        else break;
      rmchr4 := s;
    end
  else rmchr4 := '';
end;

// -- DOS shell ----------------------------------------------------------------
function dosshell(message: string): integer;
begin
  window(1,1,80,25);
  textcolor(lightgray); textbackground(black); clrscr;
  writeln(message);
  cursoron;
  swapvectors;
  exec(getenv('COMSPEC'),'/k prompt $p$g$g');
  swapvectors;
  cursoroff;
  dosshell:=doserror;
end;

end.
