{ +--------------------------------------------------------------------------+ }
{ | Tubes2 for DOS 0.2 Open edition * Electron tube catalog                  | }
{ | Copyright (C) 2016-2017 Pozsar Zsolt <info@pozsarzs.hu>                  | }
{ | uhelp.pas                                                                | }
{ | help                                                                     | }
{ +--------------------------------------------------------------------------+ }

{
   This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by the
 Free Software Foundation, either version 3 of the License, or (at your
 option) any later version.

   This program is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.

   You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
}

{$MODE OBJFPC}{$H+}
unit uhelp;
interface
uses
  crt,
  uconfig, ucommon;
var
  b, bb: byte;                                    { General byte type variable }
  c: char;                                        { General character variable }
  s, ss, x: string;                             { General string type variable }

procedure helpbox(section: string);
procedure helpcontent;

resourcestring
  MESSAGE01='<Enter> Close  <Escape> Close';
  MESSAGE02='Help';
  MESSAGE03='Help isn''t available on  your language!';
  MESSAGE04='Warning';
  MESSAGE05='<Enter> OK';
  MESSAGE06='Package information';
  MESSAGE07='Close';

implementation

// -- Remove bad characters ----------------------------------------------------
function rembadchar(input: string): string;
begin
  x:=input;
  delete(x,2,length(x));
  if x<>'#' then
  begin
    input:=rmchr3(input);     { remove all 'empty' characters to begin of text }
    input:=rmchr2(input);                   { remove all characters after hash }
  end;
  input:=rmchr1(input);                       { remove all space and tabulator }
  rembadchar:=input;
end;

// -- Create help box ----------------------------------------------------------
procedure helpbox(section: string);

function loadhlp: boolean;
var
  textbuffer: array[0..4095] Of byte;                   { Buffer for help file }
  textfile: text;                                          { General text file }
begin
  settextbuf(textfile,textbuffer);
  if section<>'p' then
    assign(textfile,exepath+'..\help\tubes_'+lng+'.hlp') else
    assign(textfile,exepath+'..\library\info_'+lng+'.txt');
  {$I-}
  reset(textfile);
  {$I+}
  if ioresult <> 0 Then
  begin
    loadhlp:=false;
    exit;
  end else
  begin
    if section<>'p' then
    repeat
      readln(textfile,s);
    until (rembadchar(s)='SECT'+section) or (eof(textfile));
    repeat
      readln(textfile,s);
      if rembadchar(s)<>'END'
        then ewrite(menu_foreground,menu_highlighted,s+#13+#10);
    until (rembadchar(s)='END') or (eof(textfile));
    close(textfile);
    loadhlp:=true;
  end;
end;

begin
  footer(MESSAGE01);
  textbackground(black); window(5,3,79,24); clrscr;
  textbackground(menu_background); textcolor(menu_foreground); win(3,2,77,23);
  gotoxy(4,1);
  if section<>'p' then write(' '+MESSAGE02+' ') else write(' '+MESSAGE06+' ');
  line(5,77,21);
  textbackground(button_background); textcolor(button_foreground);
  gotoxy(34,22); writeln('[ '+MESSAGE07+' ]');
  textbackground(menu_background); textcolor(menu_foreground); window(6,3,74,21);
  if loadhlp=false then write(MESSAGE03);
  repeat
    c:=readkey;
  until (c=#27) or (c=#13);
  c:=' ';
end;

// -- Content of help ----------------------------------------------------------
procedure helpcontent;
var
  TOC: array[1..20] Of string;
  items: itemrec;                                           { Table of content }
  messtext: messagerec;                      { Variable of messages procedures }
  textbuffer: array[0..4095] Of byte;                   { Buffer for help file }
  textfile: text;                                          { General text file }
begin
  for b:=1 to 20 do TOC[b]:='';
  settextbuf(textfile,textbuffer);
  assign(textfile,exepath+'..\help\tubes_'+lng+'.hlp');
  {$I-}
  reset(textfile);
  {$I+}
  if ioresult <> 0 then
  begin
    with messtext do
    begin
      title:=MESSAGE04;
      message:=MESSAGE03;
      ok:=' OK ';
      cancel:='';
      foot_message:=MESSAGE05;
    end;
    message_box(messtext);
    exit;
  end else
  begin
    b:=1;
    repeat
      readln(textfile,s);
      s:=rembadchar(s);
      x:=s;
      delete(x,5,length(x));
      if x='SECT' then
      begin
        readln(textfile,s);
        ss:='';
        for bb:=1 to length(s) do
          if (s[bb]<>'<') and (s[bb]<>'>') then ss:=ss+s[bb];
        TOC[b]:=rmchr3(ss)+'<>';
        b:=b+1;
      end;
    until eof(textfile);
    close(textfile);
    with items do
    begin
      i1:=TOC[1];
      i2:=TOC[2];
      i3:=TOC[3];
      i4:=TOC[4];
      i5:=TOC[5];
      i6:=TOC[6];
      i7:=TOC[7];
      i8:=TOC[8];
      i9:=TOC[9];
      i10:=TOC[10];
      i11:=TOC[11];
      i12:=TOC[12];
      i13:=TOC[13];
      i14:=TOC[14];
      i15:=TOC[15];
      i16:=TOC[16];
      i17:=TOC[17];
      i18:=TOC[18];
      i19:=TOC[19];
      i20:=TOC[20];
    end;
    b:=menu(41,6,items);
    if b=255 then exit;
    str(b,s);
    helpbox(s);
  end;
end;

end.
